import { DefinitionParams, Location, TypeDefinitionParams } from "vscode-languageserver";
import { Languages } from "@blockception/shared";
import { Console } from "../Manager";
import { Json, Mcfunction } from "../Minecraft";
import { GetDocument } from "../Types/Document/Document";

export function onDefinitionRequestAsync(params: DefinitionParams): Promise<Location[] | undefined> {
  return Console.request("Definition", Promise.resolve(onDefinition(params)));
}

export function onTypeDefinitionRequestAsync(params: TypeDefinitionParams): Promise<Location[] | undefined> {
  return Console.request("Type Definition", Promise.resolve(onDefinition(params)));
}

export function onDefinition(params: TypeDefinitionParams | DefinitionParams): Location[] | undefined {
  const doc = GetDocument(params.textDocument.uri);
  if (!doc) return undefined;

  switch (doc.languageId) {
    case Languages.McFunctionIdentifier:
      return Mcfunction.provideReferences(params, doc);

    case Languages.JsonCIdentifier:
    case Languages.JsonIdentifier:
      return Json.provideReferences(doc, params);

    case Languages.McOtherIdentifier:
      break;
  }

  return [];
}
